/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   TOYAMA Nao <nanto@moon.email.ne.jp>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function nntPerson() { /* 初期化処理 */ }

// 各プロパティ・メソッドを実装する
nntPerson.prototype = {
    // プロパティの実装はget、setキーワードを用いて行う
    get age()         { return this._age; },
    set age(newValue) { return this._age = newValue; },

    say: function ()
    {
        return "Hello, I'm " + this._age + " years old.";
    },

    // nsISupportsインターフェースの実装
    QueryInterface: function (iid)
    {
        // 実装しているインターフェースを全て列挙
        // どんなインターフェースもnsISupportsインターフェースを実装している
        if (iid.equals(Components.interfaces.nntIPerson) ||
            iid.equals(Components.interfaces.nsISupports))
            return this;

        Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
        return null;
    },

    // プライベートメンバ
    _age: 0
}

var nntPersonModule = {
    // コンポーネントの説明
    _description: "JS Person Component",

    // IDLに指定したUUIDを指定する
    _CID: Components.ID("{5E458528-CE7C-4610-9872-5177DCB32E7C}"),

    // コンポーネントを識別するための文字列
    // http://www-6.ibm.com/jp/developerworks/components/010907/j_co-xpcom2.html#4
    _contractID: "@nanto.asablo.jp/jsperson;1",

    _firstTime: true,

    registerSelf: function (compMgr, fileSpec, location, type)
    {
        if (this._firstTime) {
            this._firstTime = false;
            throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
        }
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(this._CID,
                                        this._description,
                                        this._contractID,
                                        fileSpec,
                                        location,
                                        type);
    },

    getClassObject: function (compMgr, cid, iid)
    {
        if (!cid.equals(this._CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;

        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

        return this._factory;
    },

    _factory: {
        createInstance: function (outer, iid)
        {
            if (outer != null)
                throw Components.results.NS_ERROR_NO_AGGREGATION;

            return (new nntPerson()).QueryInterface(iid);
        }
    },

    // コンポーネントがアンロードされるときに呼ばれる
    // JSコンポーネントでは常にtrueを返せばいい
    canUnload: function(compMgr)
    {
        return true;
    }
};

function NSGetModule(compMgr, fileSpec)
{
    return nntPersonModule;
}
